function commonSubmitArgs = getCommonSubmitArgs(cluster, numWorkers)
% Get any additional submit arguments for the Slurm sbatch command
% that are common to both independent and communicating jobs.

% Copyright 2016-2017 The MathWorks, Inc.

% https://research.csc.fi/taito-batch-jobs
% https://research.csc.fi/taito-constructing-a-batch-job-file

commonSubmitArgs = '';

% The following are not supported below, though could be added with
%
%   c.AdditionalProperties.AdditionalSubmitArgs
%
%     --nodes (though we support --ntasks-per-node/-n)
%     --mem (though we support --mem-per-cpu)
%

% Number of cores/node
ppn = validatedPropValue(cluster, 'ProcsPerNode', 'double');
if ~isempty(ppn) && ppn>0
    % Don't request more cores/node than workers
    ppn = min(numWorkers,ppn);
    commonSubmitArgs = sprintf('%s --ntasks-per-node=%d',commonSubmitArgs,ppn);
end
commonSubmitArgs = sprintf('%s -c %d --ntasks-per-core=1',commonSubmitArgs,cluster.NumThreads);


%% REQUIRED

% Physical Memory used by a single core
mu = validatedPropValue(cluster, 'MemUsage', 'char');
if isempty(mu)
    emsg = sprintf(['\n\t>> %% Must set MemUsage to memory per core (in MB).  E.g.\n\n', ...
                    '\t>> c = parcluster(''%s'');\n', ...
                    '\t>> %% 2 GB per core\n', ...
                    '\t>> c.AdditionalProperties.MemUsage = ''2048'';\n', ...
                    '\t>> c.saveProfile\n\n'],cluster.Profile);
    error(emsg) %#ok<SPERR>
else
    % Per CPU (in MB)
    commonSubmitArgs = [commonSubmitArgs ' --mem-per-cpu=' mu];
end

% Walltime
wt = validatedPropValue(cluster, 'WallTime', 'char');
if isempty(wt)
    emsg = sprintf(['\n\t>> %% Must set WallTime.  E.g.\n\n', ...
                    '\t>> c = parcluster(''%s'');\n', ...
                    '\t>> %% 5 hours\n', ...
                    '\t>> c.AdditionalProperties.WallTime = ''05:00:00'';\n', ...
                    '\t>> c.saveProfile\n\n'],cluster.Profile);
    error(emsg) %#ok<SPERR>
else
    commonSubmitArgs = [commonSubmitArgs ' -t ' wt];
end


%% OPTIONAL

% Partition / Check for GPU
ngpus = validatedPropValue(cluster, 'GpusPerNode', 'double');
if ngpus>0
    
    if ngpus>1
        % If running on the k80 or p100, need to request the entire node
        xflag = ' --exclusive';
    else
        xflag = '';
    end
    
    % gpu     : 3 days, up to 36 nodes
    % gpulong : 14 days, up to 10 nodes
    % gputest : 15 minutes, up to 2 nodes

    % Need:
    %
    %   type : (k80/p100)
    %   n    : (1-4)
    %   queue: (gpu/gpulong/gputest)

    gcard = validatedPropValue(cluster, 'GpuCard', 'char'); 
    if isempty(gcard)
        if ngpus>0 && ngpus<4
            gcard = 'k80';
        elseif ngpus==4
            gcard = 'p100';
        else
            error('Unsupported number of GPUs/node [1, 2, 3, or 4]: %d',ngpus)
        end            
    end
    
    qn = validatedPropValue(cluster, 'GpuQueueName', 'char'); 
    if isempty(qn)
        qn = 'gpu';
    end

    commonSubmitArgs = [commonSubmitArgs ' -p ' qn ' --gres=gpu:' gcard ':' num2str(ngpus) xflag];
    commonSubmitArgs = strrep(commonSubmitArgs,'::',':');
else
    qn = validatedPropValue(cluster, 'QueueName', 'char');
    if ~isempty(qn)
        commonSubmitArgs = [commonSubmitArgs ' -p ' qn];
    end
end

%{
% MW: --constraint is no longer supported, per email dated Tue 2/13/2018 9:16 AM
%
%   [taito-users] Taito GPU BU billing to change on February 26, 2018
%
% Architecture
a = validatedPropValue(cluster, 'Arch', 'char');
if ~isempty(a)
    commonSubmitArgs = [commonSubmitArgs ' --constraint=' a];
end
%}

% Reservation
res = validatedPropValue(cluster, 'Reservation', 'char');
if ~isempty(res)
    commonSubmitArgs = [commonSubmitArgs ' --reservation=' res];
end

% Email notification
ea = validatedPropValue(cluster, 'EmailAddress', 'char');
if ~isempty(ea)
    commonSubmitArgs = [commonSubmitArgs ' --mail-type=ALL --mail-user=' ea];
end

% Every job is going to require a certain number of MDCS licenses.
% Specification of MDCS licenses which must be allocated to this
% job.  The /etc/slurm/slurm.conf file must list
%
%   # MDCS licenses
%   Licenses=mdcs:500
%
% And then call
%
%   % scontrol reconfigure
%
commonSubmitArgs = sprintf('%s --licenses=mdcs:%d',commonSubmitArgs,numWorkers);

% Catch-all
asa = validatedPropValue(cluster, 'AdditionalSubmitArgs', 'char');
if ~isempty(asa)
    commonSubmitArgs = [commonSubmitArgs ' ' asa];
end

commonSubmitArgs = strtrim(commonSubmitArgs);
